@@ -18,7 +18,17 @@
 import sys, os, io, importlib, argparse, re, copy, getpass, datetime, traceback
 import jinja2, yaml, pytz
 
-__version__ = '1.10.0'
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
+from cryptography.hazmat.primitives.hmac import HMAC
+from cryptography.hazmat.primitives.padding import PKCS7
+from cryptography.hazmat.primitives.ciphers import Cipher
+from cryptography.hazmat.primitives.ciphers.algorithms import AES
+from cryptography.hazmat.primitives.ciphers.modes import CTR
+from cryptography.hazmat.backends import default_backend
+from cryptography.exceptions import InvalidSignature
+
+__version__ = '1.10.2'
 
 def main():
   try:
@@ -27,12 +37,8 @@ def main():
       print('Copyright (c) 2020-2022 Chris Mason <chris@netnix.org>\n')
 
     jinjafx_usage = '-t <template.j2> [-d <data.csv>] [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
-    # jinjafx_usage = '(-t <template.j2> [-d <data.csv>] | -dt <dt.yml>) [-g <vars.yml>] [-ed <exts dir>] [-o <output file>] [-od <output dir>] [-m] [-q]'
 
     parser = __ArgumentParser(add_help=False, usage='%(prog)s ' + jinjafx_usage)
-    #group_ex = parser.add_mutually_exclusive_group(required=True)
-    #group_ex.add_argument('-dt', metavar='<dt.yml>', type=argparse.FileType('r'))
-    #group_ex.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'))
     parser.add_argument('-t', metavar='<template.j2>', type=argparse.FileType('r'), required=True)
     parser.add_argument('-d', metavar='<data.csv>', type=argparse.FileType('r'))
     parser.add_argument('-g', metavar='<vars.yml>', type=argparse.FileType('r'), action='append')
@@ -43,9 +49,6 @@ def main():
     parser.add_argument('-q', action='store_true')
     args = parser.parse_args()
 
-    #if args.dt is not None and args.d is not None:
-    #  parser.error("argument -d: not allowed with argument -dt")
-
     if args.m is True and args.g is None:
       parser.error("argument -m: only allowed with argument -g")
 
@@ -53,32 +56,26 @@ def main():
       parser.error("argument -od: unable to write to output directory")
 
     data = None
-    vault = [ None ]
+    vpw = [ None ]
     gvars = {}
-    #dt = {}
 
     def decrypt_vault(string):
       if string.startswith('$ANSIBLE_VAULT;'):
-        if vault[0] is None:
-          from ansible.constants import DEFAULT_VAULT_ID_MATCH
-          from ansible.parsing.vault import VaultLib
-          from ansible.parsing.vault import VaultSecret
+        if vpw[0] is None:
+          vpw[0] = os.getenv('ANSIBLE_VAULT_PASSWORD')
 
-          vpw = os.getenv('ANSIBLE_VAULT_PASSWORD')
-
-          if vpw == None:
+          if vpw[0] == None:
             vpwf = os.getenv('ANSIBLE_VAULT_PASSWORD_FILE')
             if vpwf != None:
               with open(vpwf) as f:
-                vpw = f.read().strip()
+                vpw[0] = f.read().strip()
 
-          if vpw == None:
-            vpw = getpass.getpass('Vault Password: ')
+          if vpw[0] == None:
+            vpw[0] = getpass.getpass('Vault Password: ')
             print()
 
-          vault[0] = VaultLib([(DEFAULT_VAULT_ID_MATCH, VaultSecret(vpw.encode('utf-8')))])
+        return ansible_vault_decrypt(string, vpw[0])
 
-        return vault[0].decrypt(string.encode('utf-8')).decode('utf-8')
       return string
 
     def yaml_vault_tag(loader, node):
@@ -103,23 +100,6 @@ def main():
 
       return dst
 
-    #if args.dt is not None:
-    #  with open(args.dt.name) as f:
-    #    dt.update(yaml.load(f.read(), Loader=yaml.SafeLoader)['dt'])
-    #    args.t = dt['template']
-
-    #    if 'datasets' in dt:
-    #      raise Exception('invalid datatemplate format')
-
-    #    else:
-    #      if 'data' in dt:
-    #        data = dt['data']
-
-    #      if 'vars' in dt:
-    #        gyaml = decrypt_vault(dt['vars'])
-    #        if gyaml:
-    #          gvars.update(yaml.load(gyaml, Loader=yaml.SafeLoader))
-
     if args.d is not None:
       with open(args.d.name) as f:
         data = f.read()
@@ -249,6 +229,42 @@ class __ArgumentParser(argparse.ArgumentParser):
     raise Exception(message)
 
 
+def ansible_vault_decrypt(string, vpw):
+  slines = string.splitlines()
+  hdr = list(map(str.strip, slines[0].split(';')))
+
+  if hdr[1] == '1.1' or hdr[1] == '1.2':
+    if hdr[2] == 'AES256':
+      vaulttext = bytes.fromhex(''.join(slines[1:]))
+      b_salt, b_hmac, b_ciphertext = vaulttext.split(b"\n", 2)
+      b_salt = bytes.fromhex(b_salt.decode('utf-8'))
+      b_hmac = bytes.fromhex(b_hmac.decode('utf-8'))
+      b_ciphertext = bytes.fromhex(b_ciphertext.decode('utf-8'))
+
+      b_derivedkey = PBKDF2HMAC(hashes.SHA256(), 80, b_salt, 10000, default_backend()).derive(vpw.encode('utf-8'))
+
+      hmac = HMAC(b_derivedkey[32:64], hashes.SHA256(), default_backend())
+      hmac.update(b_ciphertext)
+
+      try:
+        hmac.verify(b_hmac)
+
+      except InvalidSignature:
+        raise Exception("invalid ansible vault password")
+
+      d = Cipher(AES(b_derivedkey[:32]), CTR(b_derivedkey[64:80]), default_backend()).decryptor()
+
+      u = PKCS7(128).unpadder()
+      b_plaintext = u.update(d.update(b_ciphertext) + d.finalize()) + u.finalize()
+      return b_plaintext.decode('utf-8')
+
+    else:
+      raise Exception("unknown ansible vault cipher")
+
+  else:
+    raise Exception("unknown ansible vault version")
+
+
 class JinjaFx():
   def jinjafx(self, template, data, gvars, output, exts_dirs=[]):
     self.__g_datarows = []
